
source ./lights.tcl

###
## intersection_example
##
##	Build the intersection and 
##	setup the when statements.
##
#
proc draw_intersection {{win .wave} args} {

	global sigList tailList
	global vsimPriv
	global PathSeparator

	# If the old one exists, delete it.
	if {[winfo exists .traffic]} {
		destroy .traffic
        }

  global tree

  # Append .tree to the window name so that we locate the signal list.
  #
	append win .tree

  set tree $win	
	
  if {![winfo exists $win]} {
    error "traffic debug: source window $win doesn't exist"
  }
  if {[string compare WaveTree [winfo class $win]]} {
    error "traffic debug: source window $win isn't a WaveTree"
  }



	toplevel .traffic
	wm geometry .traffic 785x780+2+2
	build_intersection .traffic.i
	pack .traffic.i

	create_run_buttons .traffic.i

   update_tailList $win

  set changefunc [list curs_change $win]

#   trace variable vsimPriv(acttime) {rw} check_trace

}

proc update_tailList { win } {

  global tailList sigList PathSeparator truncList selList vsimPriv

  # Find which signals are selected:
  #
  set sigList ""
  set selList [$win curselection]
  if {[llength $selList] < 1} {
    error "debug window: must select atleast 1 signal to debug"
  } else {
	set signalno [llength $selList]
	debug selected "$signalno signals"
  }
  foreach sig $selList {

# can be added if required  
#    if {[string compare [$win itemcget $sig -radix] binary]} {
#      error "argand: selected signals must have binary radix"
#    }
    
    lappend sigList [$win get4 $sig]
  }

  # Now we know the signal names, extract their tails (the
  # HDL declared name of the signal) for annotating the display
  set tailList ""
  set j 0
  foreach sig $sigList {
    lappend tailList [lindex [split $sig $PathSeparator] end]
    set j [expr $j+1]
  }
   trace variable vsimPriv(acttime) {rw} check_trace
}  


proc check_trace { varname elementname operation } {

upvar $varname var1
global vsimPriv
global tree
global sigList tailList truncList selList
set valch 1

  set k 0
  if { $valch == 1 } {
	foreach  i  $selList {
		set outp [$tree get3 $i]
		set_lights [lindex $tailList $k] $outp
		set k [expr $k+1]
	}
  }
}

proc debugwindow { obj } {
         catch {destroy .traffic.i.debug}
         label .traffic.i.debug -text " test space"
         $obj create window 18c 2.5c -window .traffic.i.debug
}

proc debug { testvar testdata } {

    .traffic.i.debug configure -text "$testvar $testdata"

}

proc close_window {obj} {

  after 100 {
#        quit -sim; noview wave
        destroy .traffic
  }     
}

proc testbreak { command1 } {

  $command1

}

proc create_run_buttons { obj } {

global tree

	set rb $obj.runall
	button $rb -command "transcribe run -all" -text "Run Forever"
	pack $rb
	$obj.c create window 17c 16c -window $rb

	set rb $obj.break
	button $rb -command "testbreak vsim_break" -text "Break"
	pack $rb
	$obj.c create window 17c 17c -window $rb

	set rb $obj.reload
	button $rb -command "update_tailList $tree" -text "Reload selected"
	pack $rb
	$obj.c create window 17c 18c -window $rb

        set rb $obj.exit
        button $rb -command "close_window $obj" -text "Exit"
        pack $rb
        $obj.c create window 17c 19c -window $rb

}

###
## create a traffic light
##
##	Given an object name, create a frame
##	with the given object name. In the frame create
##	a canvas. On the canvas draw a stop light. 
#
proc create_light_v { obj } {

    frame $obj -relief sunken -height 3c -width 1c
    canvas $obj.c -width 1c -height 3c
    pack $obj.c -fill both

    # Draw the "light bulbs"
    $obj.c create oval 0.1c 0.1c 0.9c 0.9c -fill gray -tag $obj.c.red
    $obj.c create oval 0.1c 1.1c 0.9c 1.9c -fill gray -tag $obj.c.yellow
    $obj.c create oval 0.1c 2.1c 0.9c 2.9c -fill gray -tag $obj.c.green

    return $obj
}

proc create_light_h { obj } {

    frame $obj -relief sunken -height 1c -width 3c
    canvas $obj.c -width 3c -height 1c
    pack $obj.c -fill both
  
    # Draw the "light bulbs"
    $obj.c create oval 0.1c 0.1c 0.9c 0.9c -fill gray -tag $obj.c.red
    $obj.c create oval 1.1c 0.1c 1.9c 0.9c -fill gray -tag $obj.c.yellow
    $obj.c create oval 2.1c 0.1c 2.9c 0.9c -fill gray -tag $obj.c.green
    
    return $obj
}

###
## set_light_state
##
## 	Given an object name, and a color, light up the
##	proper "light bulb" with the proper color.
##
##	"color" must be one of "red", "green", or "yellow".
##	"obj" must have been created previously with create_light
##
#
proc set_light_state {color obj} {

    # turn off the old light
  #  debug $color $obj
    switch $color {
	red {
	  $obj.c itemconfigure $obj.c.yellow -fill gray 
	  $obj.c itemconfigure $obj.c.green -fill gray
        }
	yellow {
          $obj.c itemconfigure $obj.c.green -fill gray
          $obj.c itemconfigure $obj.c.red -fill gray
        }
	green {
          $obj.c itemconfigure $obj.c.red -fill gray
          $obj.c itemconfigure $obj.c.yellow -fill gray
        }
    }

    # turn on the new light
    $obj.c itemconfigure $obj.c.$color -fill $color


    # Flush all the "events" out to the X server.
    update
}

###
## build_intersection
##
##	Given an object name, create a giant canvas, and
##	draws lines representing an intersection.
##	Create two stop lights and put them on the canvas too.
##
#
proc build_intersection { obj } {

    frame $obj -height 21c -width 21c -bg gray -relief flat
    canvas $obj.c -height 21c -width 21c -bg gray -relief flat
    #frame $obj -height 25c -width 25c -bg gray -relief flat
    #canvas $obj.c -height 25c -width 25c -bg gray -relief flat
    pack $obj.c -in $obj
    
    # draw the  lines that make up the "roads"

    # top-left quadrant
    $obj.c create line 8c 0c 8c 6c
    $obj.c create line 8c 6c 0c 6c

    # bottom-left quadrant
    $obj.c create line 0c 15c 8c 15c
    $obj.c create line 8c 15c 8c 21c

#medians
#vertical
    $obj.c create line 11c 0c 11c 6c
    $obj.c create line 11c 15c 11c 21c
#Horizontal   
    $obj.c create line 0c 9c 8c 9c
    $obj.c create line 0c 12c 8c 12c

    $obj.c create line 14c 9c 22c 9c
    $obj.c create line 14c 12c 22c 12c

    # top-right quadrant
    $obj.c create line 14c 0c 14c 6c
    $obj.c create line 14c 6c 22c 6c

    # bottom-right quadrant
    $obj.c create line 21c 15c 14c 15c
    $obj.c create line 14c 15c 14c 21c

   # embed the traffic lights into the roads
    $obj.c create window 14c 6c \
	-window [create_light_v $obj.ew_light_w] \
         -anchor nw -tags ns_light
    $obj.c create window 14c 9c \
        -window [create_light_v $obj.w_turn_light] \
         -anchor nw -tags ns_light

    $obj.c create window 7c 12c \
        -window [create_light_v $obj.ew_light_e] \
         -anchor nw -tags ns_light
    $obj.c create window 7c 9c \
        -window [create_light_v $obj.e_turn_light] \
         -anchor nw -tags ns_light

# NS lights
    $obj.c create window 11c 15c \
	-window [create_light_h $obj.ns_light_s] \
         -anchor nw -tags ns_light
    $obj.c create window 8c 5c \
        -window [create_light_h $obj.ns_light_n] \
         -anchor nw -tags ns_light


    # add intersection sign
    catch {destroy .traffic.i.sign}
	label .traffic.i.sign -text " \nUsing Tcl with ModelSim [vsimId]   \
        -- a Highway-Farmroad Intersection Debugger   \n "
	.traffic.i.c create window 10c 0.5c \
	-window .traffic.i.sign 

      # add intersection sign
    catch {destroy .traffic.i.sign1}
        label .traffic.i.sign1 -text " Left Only"
        .traffic.i.c create window 4c 10.5c \
        -window .traffic.i.sign1  
    catch {destroy .traffic.i.sign2}
        label .traffic.i.sign2 -text " Left Only"                       
        .traffic.i.c create window 18c 10.5c \
        -window .traffic.i.sign2

    debugwindow .traffic.i.c

}

# draw_intersection


