library ieee;
  use ieee.std_logic_1164.all;

entity tb_traffic is
end tb_traffic;

architecture only of tb_traffic is
  component traffic
  port (
        car_ns : in std_logic;
        car_ew : in std_logic;
        car_e_left : in std_logic;
        car_w_left : in std_logic;

        light_ns : out std_logic_vector (2 downto 0);
        light_ew : out std_logic_vector (2 downto 0);
        light_ew_turn : out std_logic_vector (2 downto 0);
        light_we : out std_logic_vector (2 downto 0);
        light_we_turn : out std_logic_vector (2 downto 0);

        ns_green_time  : TIME := 30 us; -- minimum time for
        ew_green_time  : TIME := 30 us; -- minimum time for
        left_green_time  : TIME := 30 us;       -- minimum time for
        ns_yellow_time : TIME :=  30 us; -- total time for
        ew_yellow_time : TIME :=  30 us; -- total time for
        turn_yellow_time : TIME :=  30 us;       -- total time for
        all_red_time  : TIME :=  10 us   -- total time for
        );
  end 
component;

  component queue
    generic (departure_rate : time := 5 ns);
    port (is_empty     : out std_logic;
          consume      : in  std_logic;
          arrival_rate : in  time
       );
  end component;

  signal car_ew : std_logic;
  signal car_ns : std_logic;
  signal ns_wait : integer;
  signal left_wait : integer;
  signal car_e_left : std_logic;
  signal car_w_left : std_logic;

  signal light_ew : std_logic_vector (2 downto 0);
  signal light_we : std_logic_vector (2 downto 0);
  signal light_ew_turn : std_logic_vector (2 downto 0);
  signal light_we_turn : std_logic_vector (2 downto 0);
  signal light_ns : std_logic_vector (2 downto 0);

  signal queue_e_empty, 
	 queue_w_empty, 
	 queue_n_empty, 
	 queue_s_empty : std_logic;

  signal pass_ns, 
	 pass_ew : std_logic;

  signal       ns_green_time  : TIME := 2000 us; -- minimum time for
  signal       ew_green_time  : TIME := 2000 us; -- minimum time 
  signal       left_green_time  : TIME := 2000 us;       -- minimum time 

  signal       ns_yellow_time : TIME :=  800 us; -- total time for
  signal       ew_yellow_time : TIME :=  800 us; -- total time for
  signal       turn_yellow_time : TIME :=  800 us;       -- total time for
  signal       all_red_time  : TIME :=  600 us;   -- total time for

  signal ns_arrival_rate : TIME := 40 ns;
  signal ew_arrival_rate : TIME := 40 ns;

  signal clock : std_logic;

begin  --  only

--        car_ew <= '0';


  tick : process
    begin
      clock <= '1';
      wait for 100 ns;
      clock <= '0';
      wait for 100 ns;
  end process tick;

--  pass_ew <= light_ew(0);
--  pass_ns <= light_ns(0);


  test : process
  begin
        car_ns <= '0';   
        car_e_left <= '0';
        car_w_left <= '0';

  wait for 1000 us;
       car_ns <= '1';
  wait for ew_green_time;
        car_ns <= '0';
  --      car_ew <= '1'; 
  wait for 2000 us;
        car_e_left <= '1';
        car_w_left <= '1';
  wait for 4000 us;
       car_e_left <= '0';
       car_w_left <= '0';
  --     car_ew <= '1';
  wait for 2000 us;
  --     car_ew <= '0';
       car_e_left <= '1';

  wait for 4000 us;
       car_e_left <= '0';
--	car_ew <= '1';
  wait for 2000 us;

  wait for 2000 us;
        car_ns <= '1';
  wait for 4000 us;
        car_ns <= '0';
  --      car_ew <= '1';
  wait for 4000 us;
        car_e_left <= '1';
        car_w_left <= '1';
  wait for 4000 us;
       car_e_left <= '0';
       car_w_left <= '0';
  --     car_ew <= '1';
  wait for 2000 us;
  --     car_ew <= '0';
       car_e_left <= '1';

  wait for 400 us;
       car_e_left <= '0';
  --      car_ew <= '1';
  wait for 200 us;

  end process test;

  testcar : process (clock) 
  begin
     if ( light_ew = "001") then
        car_ew <= '0';
     else
	if (light_ew'event and light_ew = "100") then
	        car_ew <= '1';
        end if;
     end if;

  end process testcar;

  traffic_light : traffic port map (car_ns, car_ew, car_e_left, car_w_left,
				light_ns, light_ew, light_ew_turn, 
				light_we, light_we_turn,
			        ns_green_time,
			        ew_green_time,  
			        left_green_time, 
			        ns_yellow_time,
			        ew_yellow_time,
			        turn_yellow_time,
			        all_red_time);

end only;
