library ieee;
  use ieee.std_logic_1164.all;
--  use ieee.std_logic_arith.all;

entity traffic is
  port (
        car_ns : in std_logic;
        car_ew : in std_logic;
        car_e_left : in std_logic;
        car_w_left : in std_logic;

        light_ns : out std_logic_vector (2 downto 0);
        light_ew : out std_logic_vector (2 downto 0);
        light_ew_turn : out std_logic_vector (2 downto 0);
        light_we : out std_logic_vector (2 downto 0);
        light_we_turn : out std_logic_vector (2 downto 0);

        ns_green_time  : TIME := 300 us;	-- minimum time for 
        ew_green_time  : TIME := 300 us;	-- minimum time for 
        left_green_time  : TIME := 300 us;	-- minimum time for 
        ns_yellow_time : TIME :=  30 us;	-- total time for 
        ew_yellow_time : TIME :=  30 us;	-- total time for 
        turn_yellow_time : TIME :=  30 us;	-- total time for 
        all_red_time  : TIME :=  10 us	-- total time for 
        );
end traffic;

architecture control of traffic is
  type traffic_state is (
		ns_green, ew_green, both_left_green, 
		all_red, w_left_red, e_left_red,  
		ns_yellow, ew_yellow,both_left_yellow, 
		e_left_green, w_left_green, e_left_yellow, w_left_yellow);

  signal curr_st       : traffic_state := ew_green;
  signal next_st       : traffic_state := ew_green;
  signal last_green_st : traffic_state := ew_green;
  signal curr_st_time  : TIME := 0 ns;

begin  --  control

light_gen : process (curr_st)
  
begin  --  process light_gen 

  case curr_st is

    when all_red =>
      light_ns <= "100";
      light_ew <= "100";
      light_we <= "100";
      light_ew_turn <= "100";
      light_we_turn <= "100";

    when ns_green =>
      light_ns <= "001";
      light_ew <= "100";
      light_ew <= "100";

    when ew_green =>
      light_ns <= "100";
      light_ew <= "001";
      light_we <= "001";
      light_ew_turn <= "100";
      light_we_turn <= "100";

    when ns_yellow =>
      light_ns <= "010";
      light_ew <= "100";
      light_we <= "100";

    when ew_yellow =>
      light_ns <= "100";
      light_ew <= "010";
      light_we <= "010";
      light_ew_turn <= "100";
      light_we_turn <= "100";

    when e_left_green =>   
      light_ns <= "100";
      light_ew_turn <= "001";
      light_we <= "100";

    when w_left_green =>
      light_ns <= "100"; 
      light_we_turn <= "001";
      light_ew <= "100"; 

    when e_left_yellow =>

      if (last_green_st = e_left_green) then
	light_ns <= "100"; 
      	light_ew_turn <= "010";
      	light_we <= "100"; 
      else
	light_ns <= "100";
	light_we <= "010";
      end if;

    when w_left_yellow =>
      if (last_green_st = w_left_green) then
	light_ns <= "100"; 
	light_we_turn <= "010";
	light_ew <= "100";
      else
	light_ns <= "100";
	light_ew <= "010";
      end if;

    when e_left_red =>
      light_ns <= "100";
      light_we <= "100";
      light_ew_turn <= "100";

    when w_left_red =>
      light_ns <= "100";
      light_ew <= "100";
      light_we_turn <= "100";

    when both_left_green =>
      light_ns <= "100"; 
      light_we_turn <= "001";
      light_ew_turn <= "001";

    when both_left_yellow =>
      light_ns <= "100";
      light_we_turn <= "010";  
      light_ew_turn <= "010";

  end case;                              --  curr_st
end process light_gen;


fsm : process 

begin  --  process fsm 

    case curr_st is

      when ew_green =>
        last_green_st <= ew_green;
        if (car_ns = '1') then
          curr_st <= ew_yellow;
          wait for ew_yellow_time;
        elsif ((car_e_left = '1') and (car_w_left = '1')) then
	  curr_st <= ew_yellow;
          wait for ew_yellow_time;      
        elsif (car_e_left = '1') then
          curr_st <= e_left_yellow;
          wait for ew_yellow_time;
        elsif (car_w_left = '1') then        
          curr_st <= w_left_yellow;        
          wait for ew_yellow_time;     
        else
--          wait on car_ns or car_e_left or car_w_left;
          wait on car_ns;
        end if;

      when ns_yellow | ew_yellow =>
        curr_st <= all_red;
        wait for all_red_time;

      when all_red =>
        if ((last_green_st = ns_green) or (last_green_st = both_left_green))  then
          curr_st <= ew_green;
          wait for ew_green_time;
        elsif ((car_e_left /= '0') and (car_w_left /= '0')) then
	  curr_st <= both_left_green;
          wait for left_green_time; 
        else
	  curr_st <= ns_green;
          wait for ns_green_time;
        end if;

      when ns_green =>
        last_green_st <= ns_green;
--        if (car_ns /= '0') then
          curr_st <= ns_yellow;
          wait for ns_yellow_time;
--        else
--          wait on car_ew;
--        end if;

     when e_left_yellow  =>
	curr_st <= e_left_red;
	wait for all_red_time;

     when w_left_yellow  =>
        curr_st <= w_left_red;
        wait for all_red_time;

     when e_left_red =>
      if (last_green_st = e_left_green) then
        curr_st <= ew_green;
        wait for ew_green_time;
      else
        curr_st <= e_left_green;
        wait for left_green_time;
      end if; 

     when w_left_red =>
      if (last_green_st = w_left_green) then
        curr_st <= ew_green;   
        wait for ew_green_time;
      else
        curr_st <= w_left_green; 
        wait for left_green_time;
      end if; 
 


     when both_left_yellow=>
        curr_st <= all_red;
        wait for all_red_time;

     when both_left_green =>
        last_green_st <= both_left_green;
        curr_st <= both_left_yellow;
	wait for turn_yellow_time;

     when e_left_green =>
        last_green_st <= e_left_green;
        curr_st <= e_left_yellow;
	wait for turn_yellow_time;
     when w_left_green =>
        last_green_st <= w_left_green;
        curr_st <= w_left_yellow;
	wait for turn_yellow_time;

    end case;                           --  curr_st
end process fsm;
end control;
