library ieee;
  use ieee.std_logic_1164.all;
  use ieee.std_logic_arith.all;
  use ieee.std_logic_unsigned.all ;
entity traffic is
  port (
        car_ns : in std_logic;
        car_ew : in std_logic;
        car_e_left : in std_logic;
        car_w_left : in std_logic;
	clock : in std_logic;

        light_ns : out std_logic_vector (2 downto 0);
        light_ew : out std_logic_vector (2 downto 0);
        light_ew_turn : out std_logic_vector (2 downto 0);
        light_we : out std_logic_vector (2 downto 0);
        light_we_turn : out std_logic_vector (2 downto 0);

        ns_green_time  : INTEGER := 300;	-- minimum time for 
        ew_green_time  : INTEGER := 300;	-- minimum time for 
        left_green_time  : INTEGER := 300;	-- minimum time for 
        ns_yellow_time : INTEGER :=  30;	-- total time for 
        ew_yellow_time : INTEGER :=  30;	-- total time for 
        turn_yellow_time : INTEGER :=  30;	-- total time for 
        all_red_time  : INTEGER :=  10	-- total time for 
        );
end traffic;

architecture control of traffic is
  type traffic_state is (
		ns_green, ew_green, both_left_green, 
		all_red, w_left_red, e_left_red,  
		ns_yellow, ew_yellow,both_left_yellow, 
		e_left_green, w_left_green, e_left_yellow, w_left_yellow);

  signal curr_st       : traffic_state := ew_green;
  signal next_st       : traffic_state := ew_green;
  signal last_green_st : traffic_state := ew_green;
  signal curr_st_time  : INTEGER := 0 ;
  signal ns_wait  : INTEGER := 0 ;
  signal left_wait  : INTEGER := 0 ;
  signal ew_count : integer := 0;
  
begin  --  control

light_gen : process (curr_st)
  
begin  --  process light_gen 

  case curr_st is

    when all_red =>
      light_ns <= "100";
      light_ew <= "100";
      light_we <= "100";
      light_ew_turn <= "100";
      light_we_turn <= "100";

    when ns_green =>
      light_ns <= "001";
      light_ew <= "100";
      light_ew <= "100";

    when ew_green =>
      light_ns <= "100";
      light_ew <= "001";
      light_we <= "001";
      light_ew_turn <= "100";
      light_we_turn <= "100";

    when ns_yellow =>
      light_ns <= "010";
      light_ew <= "100";
      light_we <= "100";

    when ew_yellow =>
      light_ns <= "100";
      light_ew <= "010";
      light_we <= "010";
      light_ew_turn <= "100";
      light_we_turn <= "100";

    when e_left_green =>   
      light_ns <= "100";
      light_ew_turn <= "001";
      light_we <= "100";

    when w_left_green =>
      light_ns <= "100"; 
      light_we_turn <= "001";
      light_ew <= "100"; 

    when e_left_yellow =>

      if (last_green_st = e_left_green) then
	light_ns <= "100"; 
      	light_ew_turn <= "010";
      	light_we <= "100"; 
      else
	light_ns <= "100";
	light_we <= "010";
      end if;

    when w_left_yellow =>
      if (last_green_st = w_left_green) then
	light_ns <= "100"; 
	light_we_turn <= "010";
	light_ew <= "100";
      else
	light_ns <= "100";
	light_ew <= "010";
      end if;

    when e_left_red =>
      light_ns <= "100";
      light_we <= "100";
      light_ew_turn <= "100";

    when w_left_red =>
      light_ns <= "100";
      light_ew <= "100";
      light_we_turn <= "100";

    when both_left_green =>
      light_ns <= "100"; 
      light_we_turn <= "001";
      light_ew_turn <= "001";

    when both_left_yellow =>
      light_ns <= "100";
      light_we_turn <= "010";  
      light_ew_turn <= "010";

  end case;                              --  curr_st
end process light_gen;


synth : process (clock,curr_st)
begin

if(rising_edge(clock)) then

    curr_st_time <= curr_st_time + 1;

end if;

if(curr_st'event) then
	curr_st_time <= 0;
end if;


end process synth;

fsm : process (clock)

begin  --  process fsm 

    case curr_st is

      when ew_green =>
	if (curr_st_time >= ew_green_time) then      
	        last_green_st <= ew_green;
        	if (car_ns = '1') then
	          curr_st <= ew_yellow;
	        elsif ((car_e_left = '1') and (car_w_left = '1')) then
		  curr_st <= ew_yellow;
        	elsif (car_e_left = '1') then
	          curr_st <= e_left_yellow;
	        elsif (car_w_left = '1') then        
        	  curr_st <= w_left_yellow;        
        	end if;
	end if;
      when ns_yellow | ew_yellow =>
	if (curr_st_time >= ns_yellow_time) then
	        curr_st <= all_red;
        	-- wait for all_red_time;
	end if;
      when all_red =>
      	if (curr_st_time >= all_red_time) then
	        if ((last_green_st = ns_green) or (last_green_st = both_left_green))  then
        	  curr_st <= ew_green;
	          -- wait for ew_green_time;
        	elsif ((car_e_left /= '0') and (car_w_left /= '0') and (ns_wait <= left_wait)) then
		  curr_st <= both_left_green;
        	  -- wait for left_green_time; 
	        else
		  curr_st <= ns_green;
	          -- wait for ns_green_time;
        	end if;
	end if;
      when ns_green =>
	if (curr_st_time >= ns_green_time) then
        	last_green_st <= ns_green;
	          curr_st <= ns_yellow;
 	end if;		
     when e_left_yellow  =>
	if (curr_st_time >= turn_yellow_time) then
		curr_st <= e_left_red;
	end if;

     when w_left_yellow  =>
 	if (curr_st_time >= turn_yellow_time) then
       		curr_st <= w_left_red;
	end if;

     when e_left_red =>
     if (curr_st_time >= all_red_time) then
	      if (last_green_st = e_left_green) then
        	curr_st <= ew_green;
	      else
        	curr_st <= e_left_green;
	      end if; 
     end if;	
     when w_left_red =>
     if (curr_st_time >= all_red_time) then     
     	 if (last_green_st = w_left_green) then
        	curr_st <= ew_green;   
	 else
        	curr_st <= w_left_green; 
	 end if; 
     end if;
     when both_left_yellow=>
     if (curr_st_time >= turn_yellow_time) then
	        curr_st <= all_red;
     end if;
     when both_left_green =>
     if (curr_st_time >= left_green_time) then
        last_green_st <= both_left_green;
        curr_st <= both_left_yellow;
     end if;	
	
     when e_left_green =>
     if (curr_st_time >= left_green_time) then
        last_green_st <= e_left_green;
        curr_st <= e_left_yellow;
     end if;	
	
     when w_left_green =>
     if (curr_st_time >= left_green_time) then
        last_green_st <= w_left_green;
        curr_st <= w_left_yellow;
     end if;	

    end case;                           --  curr_st
end process fsm;

waittimes : process (clock, car_ns, car_e_left, car_w_left)
begin

if (falling_edge(car_ns)) then
	ns_wait <= 0;
elsif (falling_edge(car_e_left) or falling_edge(car_w_left)) then
	left_wait <= 0;
elsif (rising_edge(clock)) then
	if(car_ns = '1') then
		ns_wait <= ns_wait + 1;
	end if;
	if ((car_e_left = '1') and (car_w_left = '1')) then
		left_wait <= left_wait + 1;
	end if;	
end if;
end process waittimes;

end control;
